#
# Sets general shell options and defines environment variables.
#
# Authors:
#   Sorin Ionescu <sorin.ionescu@gmail.com>
#

#
# Smart URLs
#

autoload -Uz url-quote-magic
zle -N self-insert url-quote-magic

#
# Aliases
#

alias help='run-help'
alias find='noglob find'
alias locate='noglob locate'

#
# General
#

fpath=("${0:h}/prompt" $fpath)
setopt PROMPT_SUBST   # Use variable substitution in prompt
autoload -Uz promptinit && promptinit

setopt BRACE_CCL          # Allow brace character class list expansion.
setopt CHASE_LINKS        # Resolve symbolic links to their true values on cd
setopt COMBINING_CHARS    # Combine zero-length punctuation characters (accents) with the base character.
setopt RC_QUOTES          # Allow 'Henry''s Garage' instead of 'Henry'\''s Garage'.
setopt RC_EXPAND_PARAM    # Enable array expansions
setopt REMATCH_PCRE       # Use PCRE regular expressions, if available
setopt INTERACTIVE_COMMENTS  # Allow comments denoted by # on the command line
unsetopt MAIL_WARNING     # Don't print a warning message if a mail file has been accessed.
unsetopt RM_STAR_SILENT   # ask for confirmation for `rm *` or `rm path/*`
unsetopt CORRECT          # Don't autocorrect
unsetopt NOMATCH          # Don't error if glob expression doesn't match any files

TIMEFMT="%J  %U user  %S system  %P cpu  %MM mem  %*Es total"  # Output format of time keyword
REPORTTIME=5    # Display usage statistics for commands running > 5 sec
WATCH=notme     # User login/logout reporting

[ -e /etc/zsh_command_not_found ] && source /etc/zsh_command_not_found  # provide command-not-found hook

[ -z $LS_COLORS ] && [ -x /usr/bin/dircolors ] && {
  [ -r ~/.dircolors ] && eval "$(dircolors $HOME/.dircolors)" || eval "$(dircolors)"
}

#
# Jobs
#

setopt LONG_LIST_JOBS     # List jobs in the long format by default.
setopt AUTO_RESUME        # Attempt to resume existing job before creating a new process.
setopt AUTO_CONTINUE      # Continue stopped jobs on disown
setopt NOTIFY             # Report status of background jobs immediately.
unsetopt BG_NICE          # Don't run all background jobs at a lower priority.
unsetopt HUP              # Don't kill jobs on shell exit.
setopt CHECK_JOBS         # Report on jobs when shell exit.

#
# Directory
#

setopt AUTO_CD              # Auto changes to a directory without typing cd.
setopt AUTO_PUSHD           # Push the old directory onto the stack on cd.
setopt PUSHD_IGNORE_DUPS    # Do not store duplicates in the stack.
setopt PUSHD_SILENT         # Do not print the directory stack after pushd or popd.
setopt PUSHD_MINUS          # Invert +/- in cd (cd -2 => goto 2nd dir in history)
setopt PUSHD_TO_HOME        # Push to home directory when no argument is given.
setopt AUTO_NAME_DIRS       # Auto add variable-stored paths to ~ list.
setopt MULTIOS              # Write to multiple descriptors.
#setopt EXTENDED_GLOB        # Use extended globbing syntax.
setopt CLOBBER              # Do overwrite/create files with > / >>.
setopt GLOB_DOTS            # Include dotfiles when globbing

#
# History
#

HISTFILE="${ZDOTDIR:-$HOME}/.zsh_history"  # The path to the history file.
HISTSIZE=10000                   # The maximum number of events to save in the internal history.
SAVEHIST=10000                   # The maximum number of events to save in the history file.

setopt BANG_HIST                 # Treat the '!' character specially during expansion.
setopt EXTENDED_HISTORY          # Write the history file in the ':start:elapsed;command' format.
setopt INC_APPEND_HISTORY        # Write to the history file immediately, not when the shell exits.
unsetopt SHARE_HISTORY             # Don't share history between all sessions.
setopt HIST_EXPIRE_DUPS_FIRST    # Expire a duplicate event first when trimming history.
setopt HIST_IGNORE_DUPS          # Do not record an event that was just recorded again.
setopt HIST_IGNORE_ALL_DUPS      # Delete an old recorded event if a new event is a duplicate.
setopt HIST_FIND_NO_DUPS         # Do not display a previously found event.
setopt HIST_REDUCE_BLANKS        # Reduce superfluous whitespace
setopt HIST_SAVE_NO_DUPS         # Do not write a duplicate event to the history file.
setopt HIST_VERIFY               # Do not execute immediately upon history expansion.
setopt HIST_FCNTL_LOCK           # Lock HISTFILE with fcntl
